-- Hair Planter 1.0
-- by Jarek Dukat
-- http://www.nautilus.com.pl/jarek
-- jarek@nautilus.com.pl

-- This script works in way similar to Scatter tool,
-- but it is designed to create renderable shapes
-- that looks like hair when rendered.

-- Please report any bugs or wishes by email

global hobj, hair, hair_del

utility hgrow "Hair Planter 1.0"
(
--------- utility layout
	label lab1 "Pick Object to Plant Hairs:"
	pickbutton hairobj "Pick Object" filter:validmesh width:140 height:19 tooltip:"Pick Mesh, Patch or NURBS Surface on which you want to plant Hairs"

	group "Hair Distribution"
	(
		radiobuttons hairdist labels:#("All Faces", "Selected Faces", "All Vertices", "Selected Vertices")
		spinner haircount "Hair Count: " range:[1,100000,100] type:#integer scale:10 fieldwidth:48 align:#center
	)

	group "Interactive Hair Options"
	(
		colorpicker haircol "Hair Color: " color:[215,100,0] fieldwidth:35 title:"Choose Hair Color" across:2 offset:[8,0]
		button haircol_aq "obj" height:17 width:30 tooltip:"Copy color from picked object" enabled:false align:#right
		spinner hairthck "Hair Thickness: " range:[0,100000,0.3] fieldwidth:38
	)
	
	group "Hair Grow Options"
	(
		spinner hairlng "Hair Length: " range:[1,100000,15] scale:5 fieldwidth:50
		spinner hairlngv "Len. Variation %: " range:[0,100,20] scale:2 fieldwidth:39
		spinner hairdirv "Dir. Variation %: " range:[0,100,10] scale:1 fieldwidth:39
		label hairwind "Wind strength:  X  Y  Z" align:#left across:2 offset:[0,3]
		button hairwind_r "R" height:16 width:20 align:#right tooltip:"Reset all wind values to 0"
		spinner hairwind_x range:[-500,500,0] fieldwidth:37 scale:2 across:3
		spinner hairwind_y range:[-500,500,0] fieldwidth:37 scale:2
		spinner hairwind_z range:[-500,500,0] fieldwidth:37 scale:2
		spinner hairsegs "Hair segments: " range:[1,10,1] fieldwidth:37 type:#integer offset:[0,2]
		radiobuttons hairtype labels:#("Sharp", "Smooth") enabled:false columns:2
	)

	button hairgrow "GROW" width:100 height:22 offset:[0,3] enabled:false tooltip:"Plant some Hair"
	label haircnt "Current Hair Count: 0"
	button haircut "CUT" width:65 height:22 offset:[0,3] enabled:false tooltip:"Delete current Hair set" across:2
	button hairfrz "FREEZE" width:65 height:22 offset:[0,3] enabled:false tooltip:"Prepare for new Hair set"
	
	group "About Hair Planter"
	(
		bitmap logobitmap filename:"hp_logo.jpg"
		label abwww "www.nautilus.com.pl/jarek" offset:[0,-7]
		label abmail "jarek@nautilus.com.pl" offset:[0,-3]
	)

--------- utility handlers
	on hairobj picked node do
	(
		hairobj.text = node.name
		hairgrow.enabled = true
		haircol_aq.enabled = true
		hobj = node
	)

	on hairgrow pressed do
	(
		try	tmp = hobj.name catch (hair_pick(); return())
		try if hair != undefined do tmp = hair.name catch hair = undefined
		progressstart "Growing Hairs..."
		if hair == undefined do
		(
			haircut.enabled = true
			hairfrz.enabled = true
			hairgrow.text = "GROW MORE"
			hair = splineshape prefix:(hobj.name + ".Hair") pos:hobj.pos wirecolor:hgrow.haircol.color thickness:hgrow.hairthck.value
			if hgrow.hairthck.value > 0 then hair.renderable = true
			hair.parent = hobj
		)
		addmodifier hobj (mesh_select name:"hair_temp_mod")
		case hairdist.state of
		(
			1: (
				for i = 1 to haircount.value do
				(
					grow_on_face (random 1 (getnumfaces hobj))
					if progressupdate(i * 100 / haircount.value) == false do exit
				)
			)
			2: (
				fsel = getfaceselection hobj
				if fsel.count < 1 then
				(
					messagebox "Error: No faces selected!" title:"Hair Planer Error"
					hair_none()
				)
				else
				(
					for i = 1 to haircount.value do
					(
						grow_on_face fsel[random 1 (fsel.count)]
						if progressupdate(i * 100 / haircount.value) == false do exit
					)
				)
			)
			3: (
				for i = 1 to (getnumverts hobj) do
				(
					grow_on_vertex i
					if progressupdate(i * 100 / haircount.value) == false do exit
				)
			)
			4: (
				vsel = getvertselection hobj
				if vsel.count < 1 then
				(
					messagebox "Error: No vertices selected!" title:"Hair Planer Error"
					if hair.numsplines < 1 then hair_none()
				)
				else
				(
					for i = 1 to vsel.count do
					(
						grow_on_vertex vsel[i]
						if progressupdate(i * 100 / haircount.value) == false do exit
					)
				)
			)
		)
		if hair != undefined do
		(
			updateshape hair
			haircnt.text = "Current Hair Count: " + (hair.numsplines as string)
		)
		deletemodifier hobj hobj.modifiers["hair_temp_mod"]
		progressend()
	)

	on haircut pressed do
	(
		try delete hair catch ()
		hair_none()
	)

	on hairfrz pressed do hair_none()

	on hairwind_r pressed do
	(
		hairwind_x.value = 0
		hairwind_y.value = 0
		hairwind_z.value = 0
	)

	on haircol_aq pressed do
	(
		haircol.color = hobj.wirecolor
		try hair.wirecolor = hgrow.haircol.color
		catch hair_none()
	)

	on hairsegs changed tmp do
		if tmp > 1 then hairtype.enabled = true
		else hairtype.enabled = false

	on hairdist changed tmp do
		if tmp > 2 then (haircount.enabled = false; hairsegs.enabled = false)
		else (haircount.enabled = true; hairsegs.enabled = true)

	on haircol changed tmp do
		try hair.wirecolor = hgrow.haircol.color
		catch hair_none()

	on hairthck changed tmp do
		try
		(
			hair.thickness = hgrow.hairthck.value
			if hgrow.hairthck.value > 0 then hair.renderable = true
				else hair.renderable = false
		)
		catch hair_none()

	on hgrow open do hair = undefined
)

fn validmesh obj = superClassOf obj == GeometryClass

fn grow_on_face face =
(
	f = getface hobj face
	p1 = getvert hobj f.x; p2 = getvert hobj f.y; p3 = getvert hobj f.z
	r1 = random 0.0 1.0; r2 = random 0.0 1.0
	facepos = p3 + (((p1 + ((p2 - p1)*r1))-p3)*r2)
	newhair = addnewspline hair
	lvar = #(); for a = 1 to 4 do do (lvar[a] = random -1 1) while (lvar[a] == 0)
	addknot hair newhair #corner #line facepos
	
	if hgrow.hairsegs.value == 1 then
	(
		hlen = hgrow.hairlng.value + lvar[1] * random 0 (hgrow.hairlng.value / 100 * hgrow.hairlngv.value)
		dirv = [lvar[2] * random 0.0 hgrow.hairdirv.value / 100, lvar[3] * random 0.0 hgrow.hairdirv.value / 100, lvar[4] * random 0.0 hgrow.hairdirv.value / 100]
		dirv += (getfacenormal hobj face) + [hgrow.hairwind_x.value / 100, hgrow.hairwind_y.value / 100, hgrow.hairwind_z.value / 100]
		kt = #corner; st = #line
		if hgrow.hairtype.state == 2 do (kt = #smooth; st = #curve)
		addknot hair newhair kt st [facepos.x + hlen * dirv.x, facepos.y + hlen * dirv.y, facepos.z + hlen * dirv.z]
	)
	else
	(
		hl = hgrow.hairlng.value + lvar[1] * random 0 (hgrow.hairlng.value / 100 * hgrow.hairlngv.value)
		dv = [lvar[2] * random 0.0 hgrow.hairdirv.value / 100, lvar[3] * random 0.0 hgrow.hairdirv.value / 100, lvar[4] * random 0.0 hgrow.hairdirv.value / 100] + (getfacenormal hobj face)
		dw = [hgrow.hairwind_x.value / 100, hgrow.hairwind_y.value / 100, hgrow.hairwind_z.value / 100]
		for seg = 1 to hgrow.hairsegs.value do
		(
			hlen = hl / hgrow.hairsegs.value * seg
			dirv = dv / sqrt (hgrow.hairsegs.value + 1 - seg) + dw / hgrow.hairsegs.value * seg
			addknot hair newhair #corner #line [facepos.x + hlen * dirv.x, facepos.y + hlen * dirv.y, facepos.z + hlen * dirv.z]
		)
	)
)

fn grow_on_vertex vert =
(
	v = getvert hobj vert
	newhair = addnewspline hair
	lvar = #(); for a = 1 to 4 do do (lvar[a] = random -1 1) while (lvar[a] == 0)
	hlen = hgrow.hairlng.value + lvar[1] * random 0 (hgrow.hairlng.value / 100 * hgrow.hairlngv.value)
	dirv = [lvar[2] * random 0.0 hgrow.hairdirv.value / 100, lvar[3] * random 0.0 hgrow.hairdirv.value / 100, lvar[4] * random 0.0 hgrow.hairdirv.value / 100]
	dirv += [hgrow.hairwind_x.value / 100, hgrow.hairwind_y.value / 100, hgrow.hairwind_z.value / 100]
	addknot hair newhair #corner #line v
	addknot hair newhair #corner #line [v.x + hlen * dirv.x, v.y + hlen * dirv.y, v.z + hlen * dirv.z]
)

fn hair_none =
(
	hgrow.haircut.enabled = false
	hgrow.hairfrz.enabled = false
	hgrow.hairgrow.text = "GROW"
	hair = undefined
	hgrow.haircnt.text = "Current Hair Count: 0"
)

fn hair_pick =
(
	hgrow.hairobj.text = "Pick Object"
	hgrow.hairgrow.enabled = false
	hgrow.haircol_aq.enabled = false
)